@@ -27,6 +27,7 @@ use rss::Channel;
 use simple_eyre::eyre;
 
 use crate::cache::deserialise_cached_headers;
+use crate::config::ConfigHash;
 use crate::config::{ChannelConfig, Config};
 use crate::dirs::Dirs;
 use crate::feed::{process_feed, ProcessResult};
@@ -62,10 +63,24 @@ async fn try_main() -> eyre::Result<bool> {
 
     let config = Config::read(cli.config_path)?;
 
-    // Ensure output directory exists
-    let output_dir = cli.output_path.or_else(|| config.rsspls.output.map(|ref path| PathBuf::from(path)))
-        .ok_or_else(|| eyre!("output directory must be supplied via --output or be present in configuration file"))?;
+    // Determine output directory
+    let output_dir = match cli.output_path {
+        Some(path) => Some(path),
+        None => config
+            .rsspls
+            .output
+            .map(|ref path| {
+                dirs::home_dir()
+                    .ok_or_else(|| eyre!("unable to determine home directory"))
+                    .map(|home| expand_tilde(path, home))
+            })
+            .transpose()?,
+    }
+    .ok_or_else(|| {
+        eyre!("output directory must be supplied via --output or be present in configuration file")
+    })?;
 
+    // Ensure output directory exists
     if !output_dir.exists() {
         fs::create_dir_all(&output_dir).wrap_err_with(|| {
             format!(
@@ -73,6 +88,7 @@ async fn try_main() -> eyre::Result<bool> {
                 output_dir.display()
             )
         })?;
+        info!("created output directory: {}", output_dir.display());
     }
 
     // Set up the HTTP client
@@ -110,12 +126,21 @@ async fn try_main() -> eyre::Result<bool> {
     let dirs = Arc::new(Mutex::new(dirs));
 
     // Spawn the tasks
+    let config_hash = Arc::new(config.hash.clone());
     let futures = config.feed.into_iter().map(|feed| {
         let client = client.clone(); // Client uses Arc internally
         let output_dir = output_dir.clone();
         let dirs = Arc::clone(&dirs);
+        let config_hash = Arc::clone(&config_hash);
         tokio::spawn(async move {
-            let res = process(&feed, &client, output_dir, dirs).await;
+            let res = process(
+                &feed,
+                &client,
+                ConfigHash(config_hash.as_str()),
+                output_dir,
+                dirs,
+            )
+            .await;
             if let Err(ref report) = res {
                 // Eat errors when processing feeds so that we don't stop processing the others.
                 // Errors are reported, then we return a boolean indicating success or not, which
@@ -139,6 +164,7 @@ async fn try_main() -> eyre::Result<bool> {
 async fn process(
     feed: &ChannelConfig,
     client: &Client,
+    config_hash: ConfigHash<'_>,
     output_dir: PathBuf,
     dirs: Dirs,
 ) -> Result<(), Report> {
@@ -155,9 +181,9 @@ async fn process(
         dirs.place_cache_file(&cache_filename)
             .wrap_err("unable to create path to cache file")
     }?;
-    let cached_headers = deserialise_cached_headers(&cache_path);
+    let cached_headers = deserialise_cached_headers(&cache_path, config_hash);
 
-    process_feed(client, feed, &cached_headers)
+    process_feed(client, feed, config_hash, &cached_headers)
         .await
         .and_then(|ref process_result| {
             match process_result {
@@ -204,3 +230,56 @@ pub fn version_string() -> String {
 fn version() -> &'static str {
     env!("CARGO_PKG_VERSION")
 }
+
+fn expand_tilde<P: Into<PathBuf>>(path: P, mut home: PathBuf) -> PathBuf {
+    let path = path.into();
+
+    // NOTE: starts_with only considers whole path components
+    if path.starts_with("~") {
+        if path == Path::new("~") {
+            home
+        } else {
+            home.push(path.strip_prefix("~").unwrap());
+            home
+        }
+    } else {
+        path
+    }
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    #[cfg(not(windows))]
+    fn test_home() {
+        let expanded = expand_tilde("asdf", PathBuf::from("/home/foo"));
+        assert_eq!(expanded, Path::new("asdf"));
+
+        let expanded = expand_tilde("~asdf", PathBuf::from("/home/foo"));
+        assert_eq!(expanded, Path::new("~asdf"));
+
+        let expanded = expand_tilde("~/some/where", PathBuf::from("/home/foo"));
+        assert_eq!(expanded, Path::new("/home/foo/some/where"));
+
+        let expanded = expand_tilde("~/some/where", PathBuf::from("/"));
+        assert_eq!(expanded, Path::new("/some/where"));
+    }
+
+    #[test]
+    #[cfg(windows)]
+    fn test_home_windows() {
+        let expanded = expand_tilde("asdf", PathBuf::from(r"C:\Users\Foo"));
+        assert_eq!(expanded, Path::new("asdf"));
+
+        let expanded = expand_tilde("~asdf", PathBuf::from(r"C:\Users\Foo"));
+        assert_eq!(expanded, Path::new("~asdf"));
+
+        let expanded = expand_tilde(r"~\some\where", PathBuf::from(r"C:\Users\Foo"));
+        assert_eq!(expanded, Path::new(r"C:\Users\Foo\some\where"));
+
+        let expanded = expand_tilde(r"~\some\where", PathBuf::from(r"C:\"));
+        assert_eq!(expanded, Path::new(r"C:\some\where"));
+    }
+}
